// This may look like C code, but it is really -*- C++ -*-
/*
 ************************************************************************
 *
 *			   UNT Virtual Machine
 *
 * 		         Operations on semaphores
 *
 ************************************************************************
 */

#ifndef _semaphores_h
#define _semaphores_h 1
#pragma interface

#include "processes.h"

typedef unsigned short SID;		// Semaphore ID
const SID NIL_sid = 0;			// Other SIDs are positive

class SemaphoreTable;

class Sema : public QueueLink
{
  friend class SemaphoreTable;

  PID owner;				// Process-owner
  int value;				// Current semaphore value
  Semaphore lock;			// Mutual exclusion lock
  BasicQueue waiting_list;		// Waiting list of processes

  void dispose(void);			// Inactivate the semaphore
  void purge(const PID pid);		// Purge a process pid from
					// any waiting queue, if any

public:
  Sema(void);
  ~Sema(void) {}
  PID q_owner(void) const		{ return owner; }
				// Perform P-operation. If value goes
				// negative, return FALSE and queue the
				// pcb into the internal queue
  int p(PCB& pcb);			
				// Perform V-operation. If value was negative
				// get a process from waiting list and
				// return its PID. Otherwise return NIL_pid
  PID v(void);
};

class SemaphoreTable : public DiagnosticPanel
{
  Sema * slots;
  const int nslots;

  BasicQueue freeSEMs;

public:
  SemaphoreTable(const int _nslots);
  ~SemaphoreTable(void) {}
				// Get a Semaphore given its id
  Sema& operator [] (const SID id);
				// Get a new SID or NIL if it is not
				// possible
  SID new_semaphore(const int in_value, const PID owner);
  void dump(void);		// Print info about all active semas

  int is_active(const SID id);	// Check to see if SID is active
  SID owned_by(const PID owner);// Get SID owned by a given owner
  void dispose(const SID sid);	// Dispose of a given semaphore
  void purge(const PID pid);	// Purge the process pid from a semaphore
				// queue
};

#endif
